# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-12-27 11:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pipeline", "0027_auto_20201123_1552"),
    ]

    operations = [
        migrations.AlterField(
            model_name="pipelinetemplate",
            name="create_time",
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name="创建时间"),
        ),
        migrations.AlterField(
            model_name="pipelinetemplate",
            name="edit_time",
            field=models.DateTimeField(auto_now=True, db_index=True, verbose_name="修改时间"),
        ),
    ]
