# -*- coding: utf-8 -*-
# Generated by Django 1.11.23 on 2020-02-13 07:38
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("engine", "0022_scheduleservice_multi_callback_enabled"),
    ]

    operations = [
        migrations.AddField(
            model_name="status",
            name="state_refresh_at",
            field=models.DateTimeField(null=True, verbose_name="上次状态更新的时间"),
        ),
    ]
